import logging
import typing
import uuid

from sqlalchemy.orm import exc

import shortenit.models.objects as objects


class Shortener:
    """
    Shortener Object
    """

    def __init__(self, session, configuration: dict) -> typing.NoReturn:
        """
        Initialize the Shortener object.

        :param configuration: The shortenit configuration
        """
        self.logger = logging.getLogger(self.__class__.__name__)
        self.session = session
        self.configuration = configuration
        self.length = 32
        self.check_duplicate = False
        self.upper_case = False
        self.init()

    def init(self) -> typing.NoReturn:
        """
        Initialize the shortener from the configuration.
        """
        length = self.configuration.get("id_length", 32)
        if length > 32 or length <= 8:
            self.length = 32
            self.logger.warn(
                "ID length provided is not between '8' and '32', reverting to default of '32'"
            )
        else:
            self.length = length
        self.check_duplicate = self.configuration.get("check_duplicate_id", False)
        self.upper_case = self.configuration.get("id_upper_case", False)

    def generate_short_uuid(self) -> str:
        """
        Method to generate UUID.

        :returns: A UUID.
        """
        _uuid = uuid.uuid1().hex
        if self.upper_case:
            return _uuid.upper()[-self.length :]
        return _uuid.lower()[-self.length :]

    def check_uuid(self, short_uuid: str) -> bool:
        """
        Method to check the short UUID against the database.

        :returns: True if UUID exists in the database.
        """
        try:
            _ = self.session.query(objects.Pointer).filter_by(data=short_uuid).one()
        except exc.NoResultFound:
            self.logger.debug(
                "Generated short uuid '%s' was not found in the database.", short_uuid
            )
            return False

        self.logger.warn("Generated short uuid '%s' was found in the database.")
        return True

    def generate_uuid(self) -> str:
        """
        Method to generate a UUID.
        This method will generate a UUID and check if it already exists.

        :returns: A UUID.
        """
        short_uuid = self.generate_short_uuid()
        if self.check_duplicate:
            counter = 0
            while self.check_uuid(short_uuid=short_uuid):
                if counter > 10:
                    self.logger.err(
                        "Cannot generate a new unique ID,"
                        "try to configure a longer ID length."
                    )
                    return None
                short_uuid = self.generate_short_uuid()
                counter += 1
        self.logger.debug("Returning ID: '%s'", short_uuid)
        return short_uuid
